function Camera(label, uuid) {
	this.label = label;
    this.uuid = uuid;
	
	label = function () { return this.label; }
    uuid = function () { return this.uuid; }
}

function ViewMessage(type, what, arg1, arg2) {
	this.messagetype = "view";

	this.type = typeof type !== "undefined" ? type : "";
	this.what = typeof what !== "undefined" ? what : "";
	this.arg1 = typeof arg1 !== "undefined" ? arg1 : "";
	this.arg2 = typeof arg2 !== "undefined" ? arg2 : "";
}

WebViewConnection.prototype = new WebConnection("/webview");
WebViewConnection.prototype.constructor = WebViewConnection;
WebViewConnection.parent = WebConnection.prototype;

WebViewConnection.prototype._onMessage = function (message) {
	switch (message.type) {
	case "time":
		this.onTime(message.time);
		break;
	case "state":
		this.onState(message.state, message.speed, message.errors, message.rtspport);
		break;
	case "audiospectrum":
		this.onAudioSpectrum(message.camera, message.audio);
		break;
	case "status":
		this.onStatus(message.camera, message.status);
		break;
	case "cameras": {
		var cameras = [];
		if (message.cameras !== null) {
            for (var i = 0; i < message.cameras.length; ++i) cameras.push(new Camera(message.cameras[i].label, message.cameras[i].uuid));
		}
		this.onCameras(cameras);
		break;
	}
	case "timeline":
		this.onTimeline(message.timeline);
		break;
	case "timelineupdate":
		this.onTimelineUpdate(message.timeline);
		break;
	case "zoomstarted":
		this.onZoomStarted(parseFloat(message.scale.replace(',', '.')));
		break;
	case "id":
		this.id = message.id;
		this.onId(this.id);
		break;
	case "menu":
		this.onMenu(message.xml);
		break;
	case "savevideodialog":
		this.onSaveVideoDialog(message.xml);
		break;
	case "clipprogressdialog":
		this.onVideoProgressDialog(message.xml);
		break;
	case "localizations":
		this.onLocalizations($.parseXML(message.xml));
		break;
	default: return false;
	}
	
	return true;
};

function WebViewConnection() {
	this.id = "";

	this.onId = function (id) {};
	this.onMenu = function (menu) {};
	this.onCameras = function (cameras) {};
	this.onLocalizations = function(xml) {};
	this.onTime = function (epoch) {};
	this.onState = function (state, speed, unresolvedErrors, rtspPort) {};
	this.onTimeline = function (timeline) {};
	this.onTimelineUpdate = function (timeline) {};
	this.onZoomStarted = function (scale) {};

	this.seekTo = function (epoch) { this._command("seekto", epoch); };
	this.play = function () { this._command("play"); };
	this.live = function () { this._command("live"); };
	this.stop = function () { this._command("stop"); };
	this.stepForward = function () { this._command("stepfw", 1); };
	this.stepBackward = function () { this._command("stepbw", 1); };
	this.seekNext = function () { this._command("seeknext"); };
	this.togglePlayPause = function () { this._command("toggleplaypause"); };
	this.zoomTimeline = function (count) { this._command("zoomtimeline", count); };
	this.playBackwards = function () { this._command("playbackwards"); };
	this.rewind = function () { this._command("rewind"); };
	this.fastForward = function () { this._command("fastforward"); };
	this.createClip = function (cameras, begin, end) { this._command("createclip", {cameras: cameras, begin: begin, end: end}); };
	
	this.panTilt = function (camera, pan, tilt) { this._command("ptz", {camera: camera, x: pan, y: tilt, z: 0}); };
	this.ptzStop = function (camera) { this._command("ptz", {camera: camera, x: 0, y: 0, z: 0}); };
	this.zoom = function (camera, zoom) { this._command("ptz", {camera: camera, x: 0, y: 0, z: zoom}); };
	
	this.setCameras = function (cameras) { this._set("cameras", cameras); };
	this.setAllCameras = function () { this._set("allcameras"); };
	this.setDragging = function (state) { this._set("dragging", state); };
	this.setShowStatus = function (state) { this._set("showstatus", state); };
	this.setShowAudioStatus = function (state) { this._set("showaudiostatus", state); };
	this.setRTSPCommands = function (commands) { this._set("rtspcommands", commands); };
	this.setActiveCameras = function (cameras) { this._set("activecameras", cameras); };
	
	this.getTime = function () { this._get("time"); };
	this.getCameras = function () { this._get("cameras"); };
	this.getState = function () { this._get("state"); };
	this.getTimeline = function () { this._get("timeline"); };
	this.getAbout = function () { this._get("about"); };

	this._cameraStatus = null;
	this._cameraAudioStatus = null;

	this._command = function (what, arg1, arg2) { this._send(new ViewMessage("cmd", what, arg1, arg2)); };
	this._get = function (what, arg1, arg2) { this._send(new ViewMessage("get", what, arg1, arg2)); };
	this._set = function (what, arg1, arg2) { this._send(new ViewMessage("set", what, arg1, arg2)); };
};

// WebViewImages
function WebViewImages(webVideoViewId) {
	this.webVideoViewId = webVideoViewId;
	
    this.onCameraImageBase64 = function (uuid, image) {};
    this.onCameraImage = function (uuid, image) {};
	
	this.getImages = function () { this._get("images"); };
	
	this.setInterval = function (interval) { this._set("interval", interval); };
	this.setQuality = function (quality) { this._set("quality", quality); };
    this.setSize = function (width, height) { this._set("size", width, height); };
	this.setBase64 = function (state) { this._set("base64", state); };
	this.setActiveCameras = function (cameras) { this._set("activecameras", cameras); };
		
	this._cameraOfNextImage = ""; // Sen kameran nimi, jonka kuvaa ollaan seuraavaksi vastaanottamassa (Tätä käytetään, jos kuvia lähetään binary-modessa)
	
	this._onMessage = function (message) {
		if (message.type !== "image") return;

        if (message.encoding === "base64") this.onCameraImageBase64(message.uuid, message.data);
        else this._cameraOfNextImage = message.uuid;
		return true;
	};
	
	this._onBlob = function (data) {
		// Tarkastetaan, että on asetettu indeksi, jolle tämän blobin sisältämä kuva kuuluu
		if (this._cameraOfNextImage.length === 0) return;

		this.onCameraImage(this._cameraOfNextImage, data);		
		return true;
	};

	this._get = function (what, arg1, arg2) { this._send(new ViewMessage("get", what, arg1, arg2)); };
	this._set = function (what, arg1, arg2) { this._send(new ViewMessage("set", what, arg1, arg2)); };

	this.path = "/webviewimages?webview=" + this.webVideoViewId;
};

WebViewImages.prototype = new WebConnection();
WebViewImages.prototype.constructor = WebViewImages;
WebViewImages.parent = WebConnection.prototype;

WebViewImages.prototype._onOpen = function () {
	// Poistetaan kuvien base64-enkoodaus, jos selain pystyy näyttämään binaarimuotoisia
	if (typeof window.URL !== "undefined" || typeof window.webkitURL !== "undefined") this.setBase64(false);
};
